/* 
 * CSE142, Spring 1999
 * Homework 4
 * Traffic animation

   Daniel Kim

   Section AH Peng
   
 *
 *
 */


#include "gp142.h"
#include <stdio.h>
#include <math.h>

#include <time.h>
	/* 
	* <time.h> is included so the current time can be accessed
	* to initialize the random number generator.  If you don't
	* use random numbers (and you don't have to) you can either
	* ignore this #include or delete it.
	*/

/* Logical constants */
#define TRUE    		1
#define FALSE   		0

/* 
 * For many of the GP142 functions, a line width of
 * zero means to fill the shape being drawn.  The
 * constant FILL is used in function calls when a solid
 * shape is desired.
 */
#define FILL 0


/*
 * Symbolic constants.  Change as needed.
 */

/* Menu buttons: */

/* Top-left coordinates of button palette */
#define MENU_TOP		(GP142_YMAX - 5)
#define MENU_LEFT		(-GP142_XMAX + 5)

/* Height and width of buttons in pixels */
#define BUTTON_HEIGHT  	25
#define BUTTON_WIDTH 	75

/* Number of buttons */
#define NUM_BUTTONS 	9


/* Possible locations of mouse clicks: */

/*   Mouse click in some button */
#define BUTTON_CLICK 1

/*   Mouse click elsewhere */
#define OTHER_CLICK 0


/* Coordinates of objects in the drawing */

/* y-coordinate of the top of the ground in the landscape */
#define HORIZON (- 2 * (GP142_YMAX / 10))

/* Vertical center and width of highway */
#define HIGHWAY_CENTER ((-GP142_YMAX + HORIZON) / 3)
#define HIGHWAY_WIDTH  15

/* tree dimensions */
#define TREE_HEIGHT 30
#define TREE_WIDTH  15

/* Maximum number of vehicles that can appear in the animation */
#define MAX_VEHICLES	8

/* Height and width of a single car */
#define CAR_HEIGHT 24
#define CAR_WIDTH  40

/* size of the lane separating lines */
#define LINE_SIZE	5
#define SPACE_SIZE	5

/* police car number */
#define POLICE 0

/* police car status */
#define VIOLATION	2   /* chasing a speeding car */
#define ENFORCE		1   /* standby */
#define NO_LAW		0   /* no police car */

/* car status */
#define IMPOUND 1   /* car cannot move */  
#define FREE    0   /* car can move */

/* speed limit in tics per cycle */
#define SPEED_LIMIT 30

/* distance between the police car and the speeding car
   at the start of the chase */
#define CHASE 200	

/* distance between the police car and the speeding car
   at the end of the chase */
#define ARREST 40

/*****************************************************************************

     FUNCTION PROTOTYPES

*****************************************************************************/

/* Initialize random number generator.  [Ignore or		*/
/* remove if your program doesn't use random numbers.]	*/
void initialize_random_numbers();

/* Draw the background, including the sun at coordinates sun_x and sun_y
   and cloud coordinates cloud_x and cloud_y */
void draw_landscape(int sun_x, int sun_y, int cloud_x, int cloud_y,
                    int* car_number, int* color);

/* Draw the buttons */
void draw_buttons();

/* Return kind of mouse click (in a button, etc.) */
int  classify_mouse_click(int mouse_x, int mouse_y);

/* Process button click at location x, y.  Update car speed,  */
/* direction, color, whether traffic is moving as needed,
   make arrests, and reset cars. */
void handle_button(int x, int y, int *traffic_moving, int *speed,
	int *direction, int *color, int *police_car, int car_x[MAX_VEHICLES],
	int car_y[MAX_VEHICLES], int car_speed[MAX_VEHICLES],
	int car_direction[MAX_VEHICLES], int car_color[MAX_VEHICLES],
	int car_status[MAX_VEHICLES], int* car_number, int* violator);

/* Update sun coordinates sun_x, sun_y for one tick */
void move_sun(int *sun_x, int *sun_y);

/* Update clouds coordinates cloud_x, cloud_y for one tick */
void move_cloud(int *cloud_x);

/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick. */
void move_traffic(int x[MAX_VEHICLES], int y[MAX_VEHICLES],
				int direction[MAX_VEHICLES], int speed[MAX_VEHICLES],
				int* police_car, int* violator,
                int status[MAX_VEHICLES]);

/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(int x[MAX_VEHICLES], int y[MAX_VEHICLES], 
				  int color[MAX_VEHICLES], int* police_car);

/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color);

/* Draw a single police car */
void draw_police_car(int x, int y);

/* Reset the cars */
void reset_cars(int car_x[MAX_VEHICLES], int car_y[MAX_VEHICLES],
                int car_direction[MAX_VEHICLES], int car_speed[MAX_VEHICLES],
                int car_color[MAX_VEHICLES], int car_status[MAX_VEHICLES],
                int* car_number, int* police_car, int* traffic_moving);

/* Display error message if internal error is detected. */
void something_is_wrong (void);


/*****************************************************************************

     MAIN

*****************************************************************************/

/*
 * Main program contents:
 *   - Major program variables that persist throughout execution
 *   - Event loop that processes GP142 events until the user quits,
 *     and updates variables and redraws the window as needed.
 */
int main(void)
{
	/* Event handling and buttons: */
	int  quit;				/* = "user has selected quit"				*/
	int  event;				/* the most recent GP142 event				*/
	char key_pressed;		/* last keyboard character from key event	*/
	int  mouse_x, mouse_y;	/* x and y coordinates of latest mouse event*/
	int  click_kind;		/* kind of mouse click (button, etc.)		*/
	int  car_number;	/* type of car */

	/* Information about the sun: */
	int sun_x, sun_y;			/* screen x and y coordinates */

	/* Information about the clouds: */
    int cloud_x, cloud_y;   /* screen x and y coordinates */

    /* Information about vehicles in the animation:							*/
	int car_x[MAX_VEHICLES], 
		car_y[MAX_VEHICLES];/* current position of the car					*/
	int car_direction[MAX_VEHICLES];/* current direction: +1=right, -1=left			*/
	int car_color[MAX_VEHICLES];/* color of the car								*/
	int car_speed[MAX_VEHICLES];/* speed in pixels per tick when traffic moving	*/	
	int car_status[MAX_VEHICLES];/* car status */
    int police_car;/* == VIOLATION if police car is on screen */
	int violator;/* car number that is exceeding the speed limit */
    int traffic_moving;		/* = "traffic is currently moving"				*/
	

	initialize_random_numbers();
	
	/* Initialize graphics package, clear the window,	*/
	/* turn logging off, and start animation.			*/
	GP142_open();
	GP142_logging(LOG_OFF);
	GP142_clear();
	GP142_animate(ANI_RUN);

	/* set initial location of the sun */
	sun_x = GP142_XMAX - 50;
	sun_y = GP142_YMAX - 35;

	/* set initial location of the cloud */
	cloud_x = GP142_XMAX - 25;
	cloud_y = GP142_YMAX - 50;

	/* set initial conditions for the car */
	car_number = 1;

	while (car_number < (MAX_VEHICLES / 4))
	{
		car_x[car_number] = rand() % (GP142_XMAX + 1);
		car_y[car_number] = HIGHWAY_CENTER + 2 + HIGHWAY_WIDTH;
		car_direction[car_number] = -1;
		car_speed[car_number] = 5 + rand() % 11;
		car_color[car_number] = rand() % 24;
		car_status[car_number] = FREE;
		
        car_number++;
	}
	
	while (car_number < (MAX_VEHICLES / 2))
	{
		car_x[car_number] = rand() % (GP142_XMAX);
		car_y[car_number] = HIGHWAY_CENTER + 2;
		car_direction[car_number] = -1;
		car_speed[car_number] = 5 + rand() % 11;
		car_color[car_number] = rand() % 24;
        car_status[car_number] = FREE;

		car_number++;
	}

	while (car_number < (MAX_VEHICLES * 3/4))
	{
		car_x[car_number] = rand() % (GP142_XMAX);
		car_y[car_number] = HIGHWAY_CENTER + 2 - HIGHWAY_WIDTH;
		car_direction[car_number] = +1;
		car_speed[car_number] = 5 + rand() % 11;
		car_color[car_number] = rand() % 24;
        car_status[car_number] = FREE;

		car_number++;
	}

	while (car_number < MAX_VEHICLES)
	{
		car_x[car_number] = rand() % (GP142_XMAX);
		car_y[car_number] = HIGHWAY_CENTER + 2 - (2 * HIGHWAY_WIDTH);
		car_direction[car_number] = +1;
		car_speed[car_number] = 5 + rand() % 11;
		car_color[car_number] = rand() % 24;
        car_status[car_number] = FREE;

		car_number++;
	}

	car_number = 1;
	police_car = NO_LAW;
	traffic_moving = TRUE;
		
	/* Draw initial landscape, buttons, and traffic */
	draw_landscape(sun_x,sun_y,cloud_x,cloud_y, &car_number,
        &car_color[car_number]);
	draw_buttons( );
	draw_traffic(car_x, car_y, car_color, &police_car);
	

	/*
	 * Main event loop:
	 * ---- ----- -----
	 * Wait for the next user action, decode it, and call an
	 * appropriate function to handle it.  Repeat until the
	 * user selects quit.
	 */
	quit = FALSE;
	while (!quit) {
		
		/* get next event */
		event = GP142_await_event(&mouse_x, &mouse_y, &key_pressed);
		
		/* perform appropriate action depending on kind of event */
		switch (event) {
		case GP142_QUIT:
			/* Quit selected */
			quit = TRUE;
			break;
			
		case GP142_KBD:
			/* Key pressed.  Not used now.  If the program should		*/
			/* react to keyboard events, insert something appropriate	*/
			/* here.  Note that scanf CANNOT be used with GP142.		*/
			car_number++;
			if (car_number > (MAX_VEHICLES - 1))
				car_number = 1;
			break;
			
		case GP142_MOUSE:
			/* Mouse click.  Determine if in a menu button	*/
			/* or elsewhere and process accordingly.		*/
			click_kind = classify_mouse_click(mouse_x, mouse_y);
			if (click_kind == BUTTON_CLICK) {
				handle_button(mouse_x, mouse_y, &traffic_moving, 
					&car_speed[car_number],&car_direction[car_number],
					&car_color[car_number], &police_car, car_x, car_y,
					car_speed, car_direction, car_color, car_status,
					&car_number, &violator);
			} 
			else {
				/* insert code here (and additional else-if statements) */
				/* to react to mouse clicks elsewhere on the screen */
			}
			break;
			
		case GP142_PERIODIC:
			/* Timer interval event.  Update everything	*/
			/* that needs to change as time advances.	*/
			move_sun(&sun_x, &sun_y);
            move_cloud(&cloud_x);
			if (traffic_moving)
				move_traffic(car_x, car_y, car_direction, car_speed,
					&police_car, &violator, car_status);
			break;
			
		default:
			/* unknown event */
			break;
			
		}  /*end switch */
		
		/* Redraw everything */
		draw_landscape(sun_x, sun_y,cloud_x,cloud_y,&car_number,
            &car_color[car_number]);
		draw_buttons();
		draw_traffic(car_x, car_y, car_color, &police_car);
		
	}  /* end event loop */
	
	
	/* Termination: shut down graphics window and exit */
	GP142_close();
	return 0;
}  /*end main */


/*****************************************************************************

     BACKGROUND LANDSCAPE

*****************************************************************************/


/* Draw the background, including the sun at coordinates sun_x and sun_y */
void draw_landscape(int sun_x, int sun_y,int cloud_x,int cloud_y, 
                    int* car_number, int* color)
{
	int text_offset = -GP142_XMAX + 15;	/* horizontal offset of text */
	int text_size   = 12;				/* size of text font */
	int line_x;	/* position of lines separating the lanes */
	int tree_x; /* position of the trees */
    
	/* Draw the sky */
	GP142_rectangleXY(ICE_BLUE, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, GP142_YMAX, FILL);
	
	/* Draw the sun */
	GP142_circleXY(YELLOW, sun_x, sun_y , 20);
	
	/* Draw clouds */
	GP142_ovalXY(WHITE,
        cloud_x+20, cloud_y+10,
        cloud_x-20, cloud_y-10,        
        FILL);

    /* Draw the ground, highway, and text legend */
	GP142_rectangleXY(BROWN, -GP142_XMAX, -GP142_YMAX,
		GP142_XMAX, HORIZON, FILL);
	GP142_rectangleXY(MED_GRAY, 
					-GP142_XMAX, HIGHWAY_CENTER - 2*HIGHWAY_WIDTH,
					 GP142_XMAX, HIGHWAY_CENTER + 2*HIGHWAY_WIDTH,
					 FILL);
	GP142_lineXY(YELLOW,
				-GP142_XMAX, HIGHWAY_CENTER + 1,
				 GP142_XMAX, HIGHWAY_CENTER + 1,
				 1);
	GP142_lineXY(YELLOW,
				-GP142_XMAX, HIGHWAY_CENTER - 1,
				GP142_XMAX, HIGHWAY_CENTER - 1,
				1);
	
	line_x = -GP142_XMAX;
	
	while (line_x < GP142_XMAX)
	{
		GP142_lineXY(WHITE,
					line_x, HIGHWAY_CENTER + HIGHWAY_WIDTH,
					line_x + LINE_SIZE, HIGHWAY_CENTER + HIGHWAY_WIDTH,
					1);
		line_x = line_x + LINE_SIZE + SPACE_SIZE;
	}
	
	line_x = -GP142_XMAX;

	while (line_x < GP142_XMAX)
	{
		GP142_lineXY(WHITE,
					line_x, HIGHWAY_CENTER - HIGHWAY_WIDTH,
					line_x + LINE_SIZE, HIGHWAY_CENTER - HIGHWAY_WIDTH,
					1);
		line_x = line_x + LINE_SIZE + SPACE_SIZE;
	}

	GP142_printfXY (BLACK, text_offset, -GP142_YMAX+30, text_size,
		"Click a button to control the car.");
	GP142_printfXY (RED, text_offset, -GP142_YMAX+50, text_size,
        "Speeding cars will be ARRESTED!");
	
	/*Draw some mountains */
	GP142_triangleXY(MED_GRAY,
		-GP142_XMAX, HORIZON,
		GP142_XMAX, HORIZON,
		0, (4 * GP142_YMAX)/5 ,
		FILL);
	
	GP142_triangleXY(OLIVE,
		-( 5 * GP142_XMAX/4), HORIZON,
		(GP142_XMAX)/3  , HORIZON,
		-(3* GP142_XMAX)/4, (2 * GP142_YMAX)/3 ,
		FILL);
	
	GP142_triangleXY(FOREST_GREEN,
		-GP142_XMAX / 3 , HORIZON,
		GP142_XMAX, HORIZON,
		GP142_XMAX, (2 * GP142_YMAX)/3 ,
		FILL);
	
    /* draw trees */
    tree_x = -GP142_XMAX + 15;
    while (tree_x < GP142_XMAX)
    {
        GP142_triangleXY(GREEN,
            tree_x, HORIZON + TREE_HEIGHT / 4,
            tree_x + TREE_WIDTH, HORIZON + TREE_HEIGHT / 4,
            tree_x + TREE_WIDTH / 2, HORIZON + TREE_HEIGHT,
            FILL);
        GP142_rectangleXY(BROWN,
            tree_x + TREE_WIDTH/4, HORIZON,
            tree_x + TREE_WIDTH*3/4, HORIZON + TREE_HEIGHT/4,
            FILL);
        tree_x = tree_x + 50;
    }
    
    
    /* draw speed limit sign */
    GP142_rectangleXY(WHITE,
        GP142_XMAX - 40, HIGHWAY_CENTER + 2*HIGHWAY_WIDTH,
        GP142_XMAX - 45, HIGHWAY_CENTER + 2*HIGHWAY_WIDTH + 40,
        FILL);
    GP142_rectangleXY(WHITE,
        GP142_XMAX - 30, HIGHWAY_CENTER + 2*HIGHWAY_WIDTH + 40,
        GP142_XMAX - 55, HIGHWAY_CENTER + 2*HIGHWAY_WIDTH + 70,
        FILL);
    GP142_printfXY(BLACK,GP142_XMAX - 52, 
        HIGHWAY_CENTER + 2*HIGHWAY_WIDTH + 47, 20, "65");
    GP142_printfXY(BLACK,GP142_XMAX - 48,
        HIGHWAY_CENTER + 2*HIGHWAY_WIDTH + 42, 6, "MPH");

    /* instructions */
    GP142_printfXY(BLACK, 0, GP142_YMAX - 30, 12,
        "Press any key to change cars:");
    
    /* car number selected */
    GP142_printfXY(*color, GP142_XMAX - 100, GP142_YMAX - 35,
        20, "CAR %d", *car_number);

} /*end draw_landscape */



/*****************************************************************************

     BUTTONS AND MOUSE CLICKS

*****************************************************************************/


/* Draws the buttons */
void draw_buttons( )
{
	int i;
	int text_offset = MENU_LEFT+10;	/* offset of button text in button */
	int font_size = 12;				/* size of button text */
	
	/* draw rectangles for each button */
	for(i = 0; i < NUM_BUTTONS; i++) {
		GP142_rectangleXY(BLACK,
			MENU_LEFT,
			MENU_TOP - i * BUTTON_HEIGHT,
			MENU_LEFT + BUTTON_WIDTH,
			MENU_TOP - (i + 1)*BUTTON_HEIGHT, 1);
	}
	
	/* display button text */
	i = MENU_TOP - (BUTTON_HEIGHT/2) - (font_size/2);
	GP142_printfXY(FOREST_GREEN, text_offset, i, font_size, "Go");	
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(RED, text_offset, i, font_size, "Stop");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Faster");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Slower");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Reverse");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "New Color");
	
	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Arrest");

	i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Anarchy!");

    i -= BUTTON_HEIGHT;
	GP142_printfXY(BLACK, text_offset, i, font_size, "Reset");

}

/* Classify mouse click at (mouse_x,mouse_y) and return kind */
int  classify_mouse_click(int mouse_x, int mouse_y)
{
	if ((mouse_x > MENU_LEFT) &&
		(mouse_x < MENU_LEFT + BUTTON_WIDTH) &&
		(mouse_y < MENU_TOP) &&
		(mouse_y > (MENU_TOP - NUM_BUTTONS*BUTTON_HEIGHT)))
		return BUTTON_CLICK;
	else
		return OTHER_CLICK;
}



/* Process button click at location x, y.  Update car speed,	*/
/* direction, color, and whether traffic is moving as needed.	*/
void handle_button(int x, int y, int *traffic_moving, int *speed,
	int *direction, int *color, int *police_car, int car_x[MAX_VEHICLES],
	int car_y[MAX_VEHICLES], int car_speed[MAX_VEHICLES],
	int car_direction[MAX_VEHICLES], int car_color[MAX_VEHICLES],
	int car_status[MAX_VEHICLES], int* car_number, int* violator)
{
	int buttonNum;		/* number of selected button */
    int i;  /* loop counter for anarchy button */

	/* Complain if x coordinate couldn't be inside a button */
	if (x < MENU_LEFT || x > MENU_LEFT + BUTTON_WIDTH) {
		something_is_wrong();
		return;
	}
	
	/* Convert y coordinate to button number from top. */
	buttonNum = (MENU_TOP - y)/BUTTON_HEIGHT;	 
	
	switch(buttonNum) {
	case 0:
		/* go */
		*traffic_moving = TRUE;
		break;
	case 1:
		/* stop */
		*traffic_moving = FALSE;
		break;
		
	case 2:
		/* speed up and check to see if the chosen car is speeding
		   and adjust the speed of the police car accordingly */
		car_speed[*car_number] = car_speed[*car_number] + 1;
		if (*car_number == *violator)
			car_speed[POLICE] = car_speed[POLICE] + 1;
		break;
	case 3:
		/* slow down */
		*speed = *speed - 1;
		if (*speed < 0)
			*speed = 0;
		break;
		
	case 4:
		/* reverse */
		*direction = - *direction;
		break;
	case 5:
		/* change car color */
		*color = (*color+1) % MAX_COLORS;
		break;
	case 6:
		/* trafic laws enforced */
		*police_car = ENFORCE;
		break;
	case 7:
		/* cars violate traffic laws */
		*police_car = NO_LAW;
        for (i=1;i<MAX_VEHICLES;i++)
        {   
            car_speed[i] = SPEED_LIMIT + 1 + rand()%20;
            car_direction[i] = rand()%2;
            if (car_direction[i] == 0)
                car_direction[i] = 1;
            else car_direction[i] = -1;
        }
        break;
    case 8:
        /* reset cars*/
        reset_cars(car_x, car_y, car_direction, car_speed, car_color,
            car_status, car_number, police_car, traffic_moving);
        break;
    default:
		/*Bad news if this is ever reached! */
		something_is_wrong();
		break;
	}
}



/*****************************************************************************

     TRAFFIC AND OTHER ANIMATED OBJECTS

*****************************************************************************/


/* Update sun coordinates sun_x, sun_y by one tick */
void move_sun(int *sun_x, int *sun_y) {
	/* Move the sun to the left.  If it reaches the edge, move it */
	/* to the other side and down a bit. */
	(*sun_x) --;
	if (*sun_x < -GP142_XMAX -30) {
		*sun_x = - *sun_x;
		(*sun_y) --;
	}
}

/* Update cloud coordinates cloud_x by one tick */
void move_cloud(int *cloud_x) {
	/* Move the sun to the left.  If it reaches the edge, move it */
	/* to the other side and down a bit. */
	*cloud_x = *cloud_x - 2;
	if (*cloud_x < -GP142_XMAX -30) {
        *cloud_x = - *cloud_x;
    }
}

/* Update all vehicle x, y coordinates, given current x, y */
/* location, direction (+1 or -1), and speed in pixels per tick, */
void move_traffic(int x[MAX_VEHICLES], int y[MAX_VEHICLES],
                int direction[MAX_VEHICLES], int speed[MAX_VEHICLES],
				int *police_car, int *violator,
                int status[MAX_VEHICLES])
{
	/* Note: If there is more than one vehicle, this should */
	/* be updated to adjust all vehicle coordinates */

	int trial_x,/* new value for x coordinate */
		i;	/* loop counter */	

	/* move vehicle.  If it moves off the edge, reverse direction */
	
	if (*police_car == VIOLATION)
		i = 0;
	else
		i = 1;
	
	while (i<MAX_VEHICLES)
	{
		

		if (status[i] != IMPOUND)
        {
            trial_x = x[i] + (direction[i] * speed[i]);
		
		    if (trial_x > GP142_XMAX)
			    x[i] = -GP142_XMAX;
		    else if (trial_x < -GP142_XMAX)
			    x[i] = GP142_XMAX;
			    else
				    x[i] = trial_x;
        }

		/* check the speed of each car */ 
		/* draw police car if speed exceeds the speed limit */
		if ((*police_car == ENFORCE) && (status[i] != IMPOUND) &&
			((speed[i] > SPEED_LIMIT) ||
			(y[i] > HIGHWAY_CENTER && direction[i] != -1) ||
			(y[i] < HIGHWAY_CENTER && direction[i] != 1)))
		{
		    *police_car = VIOLATION;
            x[POLICE] = x[i] - CHASE * direction[i];
			y[POLICE] = y[i];
			speed[POLICE] = speed[i] + 5;
			direction[POLICE] = direction[i];
		    *violator = i;
        }
		i++;
	}
	
	/* pull the car over */
	if (*police_car == VIOLATION)
	{
		if (abs(x[POLICE] - x[*violator]) <= ARREST)
		{   
			speed[POLICE] = speed[POLICE] - speed[*violator]/4;
			speed[*violator] = speed[*violator] - speed[*violator]/4;
		}
		if (abs(x[POLICE] - x[*violator])  <= (ARREST/4))
			speed[*violator] = 0;
		
		/* reverse police if car reverses directions */
		if (x[*violator] < x[POLICE])
			direction[POLICE] = -1;
		else
			direction[POLICE] = 1;
		
		if (speed[*violator] == 0)
		{
			*police_car = ENFORCE;
			status[*violator] = IMPOUND;
			y[*violator] = -GP142_YMAX + 10;
		}
		
		
	}
}


/* Draw all vehicles given their current x, y coordinates and color */
void draw_traffic(int x[MAX_VEHICLES], int y[MAX_VEHICLES],
				  int color[MAX_VEHICLES], int* police_car)
{
	/* Note: needs to be changed if there is more than */
	/* one vehicle */
	int i;
	for(i=1;i<MAX_VEHICLES;i++)
	  draw_car(x[i],y[i],color[i]);
	if (*police_car == VIOLATION)
		draw_police_car(x[POLICE],y[POLICE]);
}



/* Draw a single car of the given color with horizontal */
/* center x and bottom y*/
void draw_car(int x, int y, int color)
{
	GP142_rectangleXY(color, x - CAR_WIDTH/2, y + CAR_HEIGHT/4,
						x + CAR_WIDTH/2, y + CAR_HEIGHT*3/4, FILL);
	GP142_rectangleXY(color, x - CAR_WIDTH/6, y + CAR_HEIGHT*3/4,
						x + CAR_WIDTH/6, y + CAR_HEIGHT, FILL);
	GP142_circleXY(BLACK, x - CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
	GP142_circleXY(BLACK, x + CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
}

/* Draw a police car */
void draw_police_car(int x, int y)
{
  GP142_rectangleXY(BLACK, x - CAR_WIDTH/2, y + CAR_HEIGHT/4,
    x - CAR_WIDTH/4, y + CAR_HEIGHT*3/4, FILL);
  GP142_rectangleXY(BLACK, x + CAR_WIDTH/4, y + CAR_HEIGHT/4,
    x + CAR_WIDTH/2, y + CAR_HEIGHT*3/4, FILL);
  GP142_rectangleXY(WHITE, x - CAR_WIDTH/4, y + CAR_HEIGHT/4,
    x + CAR_WIDTH/4, y + CAR_HEIGHT, FILL);
  GP142_circleXY(BLACK, x - CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
  GP142_circleXY(BLACK, x + CAR_WIDTH/4, y + CAR_HEIGHT/4, CAR_HEIGHT/4);
  GP142_rectangleXY(BLACK, x - CAR_WIDTH/16, y + CAR_HEIGHT/4,
    x + CAR_WIDTH/16, y + CAR_HEIGHT, FILL);
  GP142_rectangleXY(RED, x - CAR_WIDTH/16, y + CAR_HEIGHT,
    x + CAR_WIDTH/16, y + CAR_HEIGHT*5/4, FILL);
}

/* reset the cars */
void reset_cars(int car_x[MAX_VEHICLES], int car_y[MAX_VEHICLES],
                int car_direction[MAX_VEHICLES], int car_speed[MAX_VEHICLES],
                int car_color[MAX_VEHICLES], int car_status[MAX_VEHICLES],
                int* car_number, int* police_car, int* traffic_moving)
/* set initial conditions for the car */
{
	*car_number = 1;

	while (*car_number < (MAX_VEHICLES / 4))
	{
		car_x[*car_number] = rand() % (GP142_XMAX + 1);
		car_y[*car_number] = HIGHWAY_CENTER + 2 + HIGHWAY_WIDTH;
		car_direction[*car_number] = -1;
		car_speed[*car_number] = 5 + rand() % 11;
		car_color[*car_number] = rand() % 24;
		car_status[*car_number] = FREE;
		
        *car_number = *car_number + 1;
	}
	
	while (*car_number < (MAX_VEHICLES / 2))
	{
		car_x[*car_number] = rand() % (GP142_XMAX);
		car_y[*car_number] = HIGHWAY_CENTER + 2;
		car_direction[*car_number] = -1;
		car_speed[*car_number] = 5 + rand() % 11;
		car_color[*car_number] = rand() % 24;
        car_status[*car_number] = FREE;

		*car_number = *car_number + 1;
	}

	while (*car_number < (MAX_VEHICLES * 3/4))
	{
		car_x[*car_number] = rand() % (GP142_XMAX);
		car_y[*car_number] = HIGHWAY_CENTER + 2 - HIGHWAY_WIDTH;
		car_direction[*car_number] = +1;
		car_speed[*car_number] = 5 + rand() % 11;
		car_color[*car_number] = rand() % 24;
        car_status[*car_number] = FREE;

		*car_number = *car_number + 1;
	}

	while (*car_number < MAX_VEHICLES)
	{
		car_x[*car_number] = rand() % (GP142_XMAX);
		car_y[*car_number] = HIGHWAY_CENTER + 2 - (2 * HIGHWAY_WIDTH);
		car_direction[*car_number] = +1;
		car_speed[*car_number] = 5 + rand() % 11;
		car_color[*car_number] = rand() % 24;
        car_status[*car_number] = FREE;

		*car_number = *car_number + 1;
	}

	*car_number = 1;
	*police_car = NO_LAW;
	*traffic_moving = TRUE;    
}
/*****************************************************************************

     OTHER

*****************************************************************************/


/* Initialize random number generator */
/* [Ignore or delete if your program  */
/* doesn't user random numbers.] */
void initialize_random_numbers() {
	time_t t;			/* current time */
	srand (time(&t));
}



/* Display error message if internal error is detected. */
void something_is_wrong (void) {
	GP142_printfXY (RED, 0, 0, 16, "BAAAADDD NEWS!");
}